;Mini-Shell Enhanced by Chicane
;version 1.2
#include "asm86.h"
#include "ti86asm.inc"
#include "ti86ops.inc"

_ldhlz		          	equ     $437B   ;write b zeros to (hl)
_load_ram_ahl			equ		$462F
_jforce					equ		$409C
_pause					equ		$4D43
_hldiv10              	equ     $4044   ;divide hl by 10
_homeup         		equ     $4A95	;resets _curRow etc.
_dispahl                equ     $4a33	;displays ahl decimal
_get_word_ahl			equ		$521d
_pop_OP1 				equ 	$47B7
_push_OP1 				equ 	$4813
_PrevPrgm				equ		$514F
_NextPrgm				equ		$514B
_exec_basic				equ		$4C57
_exec_asm				equ		$5730
_runoff                 equ     $4AB1
_runon                  equ     $4AAD
_checkbasic				equ		$46AB
_checkasm               equ     $46AF
_off                    equ     $4101

.org _asm_exec_ram
ASMstart:
	call    _runoff				;turn busy indicator off
	call    _clrScrn			;clear the screen
KillMSE:
	call	_push_OP1
	ld		hl,VatPtr
	rst		20h
	rst		10h
	ld		(hl),$0C
	call	_pop_OP1

Display:
	ld      hl,$0001			;duh
	ld      (_penCol), hl		;set up where to put text
	ld      hl,PrgmName			;load the text address
	call    _vputs				;output in variable width text format.
	
	ld		b,57
	ld		hl,$FC75
	ld		a,%01000000
Line:
	ld		(hl),a
	ld		de,$10
	add		hl,de
	djnz	Line

	set		3,(iy+5)
	set		1,(iy+5)
	ld      hl,$082B
	ld      (_penCol),hl
	ld		hl,Name_Str
	call	_vputs
	ld      hl,$162B
	ld      (_penCol),hl
	ld		hl,Size_Str
	call	_vputs
	ld		hl,$242B
	ld      (_penCol),hl
	ld		hl,FreeMem_Str
	call	_vputs
	res		3,(iy+5)
	res		1,(iy+5)
	ld		hl,$2B2B
	ld      (_penCol),hl
	call	_MEMCHK
	call	DispHL
	ld      hl,$FC00	
	call    Invert
	
DispSomeProgs:
	ld		hl,AbsFirst			;Load address of the first posible program name into hl
	rst		20h					;Move 10 bytes at hl to OP1
	xor		a
	ld		(Pos),a
	ld		(Items),a
	ld		b,$08
Looper:
	push	bc
	xor		a
	call	_NextPrgm
	jr		c,NoNext
	ld		hl,Pos
	inc		(hl)
	ld		hl,Items
	inc		(hl)
	call	DisplayHandler
	pop		bc
	dec		b
	ld		a,$00
	cp		b
	jr		nz,Looper
	jp		Setup

NoNext:
	pop		bc
	ld		hl,Items
	xor		a
	cp		(hl)
	jr		nz,Setup
NoPrograms:
	call	_clrScrn			;clear somthing
	call	_homeup				;reset _curRow etc
	ld		hl,NoProgs			;load the string into hl
	call	_puts				;output it
	jp		RestoreMSE

Setup:
	ld		hl,AbsFirst			;Load address of the first posible program name into hl
	rst		20h					;Move 10 bytes at hl to OP1
	xor		a
	call	_NextPrgm
	ld		a,$01
	ld		(Pos),a
	call	DisplayHandler
	call	Descriptions
	call	InvertCursor

GetKey1:
	call	_getky
	cp      K_EXIT
	jp      z,Exit
	cp      K_UP
	jp      z,PrevProgram
	cp      K_DOWN
	jp      z,NextProgram
	cp      K_CLEAR
	jr      z,ShutOff
	cp		K_SECOND
	jr		z,ExecPrgm
	cp		K_ENTER
	jr		z,ExecPrgm
	cp		K_DEL
	jr		z,Delete
	cp		K_PLUS
	jr		z,ContrastUp
	cp		K_MINUS
	jr		z,ContrastDn
	jr      GetKey1

;***************************Utility Routines*****************************

ContrastUp:
    ld		a,(CONTRAST)
    cp		$1f
    jr		z,GetKey1
    inc		a
    ld		(CONTRAST),a
    out		(2),a
    jr		GetKey1

ContrastDn:
    ld		a,(CONTRAST)
	cp		$00
    jr		z,GetKey1
    dec		a
    ld		(CONTRAST),a
    out		(2),a
    jr		GetKey1

ShutOff:						;shut off routine
	res 	3,(iy+8)			;Lock the APD		
	set 	4,(iy+8)			;Expire APD
	call 	_off				;Turn It Off
	jr      GetKey1 			;Restart

ExecPrgm:
	call	RestoreMSE
	rst		10h					;find the address of the program in OP1
	call	_checkbasic			;check for asm of basic
	jr		nz,ExecBasic		;if basic, jump down
	call	_exec_asm 			;else, exec it.
	jp		ASMstart

ExecBasic:
	call	_runon				;turn busy indicator on
	call	_exec_basic			;exececute basic program
	jp		ASMstart

Delete:
	rst		10h
	call	_delvar
	jp		ASMstart

;***********************Move Up Routines*************************
PrevProgram:
	xor		a
	call	_PrevPrgm
	jp		c,GetKey1
	
MoveUp:
	call    InvertCursor
	ld      hl,Pos
	ld      a,$01
	cp      (hl)
	call    z,NoMoveUp
	dec     (hl)
	call	DisplayHandler
	call	Descriptions
	call	InvertCursor
	jp		GetKey1

NoMoveUp:
	push	hl
	ld		hl,$FF84
	ld		de,$FFF4
	ld		c,48
ScrollScreenDn:
	call	ScrollDn
	push	de
	ld		de,-11
	add		hl,de
	pop		de
	push	hl
	ex		de,hl
	ld		de,-11
	add		hl,de
	ex		de,hl
	pop		hl
	dec		c
	ld		a,c
	cp		0
	jr		nz,ScrollScreenDn
	jr		EndScrollDn
ScrollDn:
	ld		b,5
LooperDn:
	ld		a,(hl)
	ld		(de),a
	dec		hl
	dec		de
	djnz 	LooperDn
	ret

EndScrollDn:
	pop		hl
	inc		(hl)
	ret

;**************************Move Down Routines**************************

NextProgram:
	xor		a
	call	_NextPrgm
	jp		c,GetKey1
	
MoveDown:
	call    InvertCursor
	ld      hl,Pos
	ld      a,(Items)
	cp      (hl)
	ld		a,$01
	call    z,NoMoveDown
	inc     (hl) 
	call	DisplayHandler
	call	Descriptions
	call	InvertCursor
	jp		GetKey1

NoMoveDown:
	push	hl
	ld		hl,$FCF0
	ld		de,$FC80
	ld		c,48

ScrollScreenUp:
	call	ScrollUp
	push	de
	ld		de,11
	add		hl,de
	pop		de
	push	hl
	ex		de,hl
	ld		de,11
	add		hl,de
	ex		de,hl
	pop		hl
	dec		c
	ld		a,c
	cp		0
	jr		nz,ScrollScreenUp
	jr		EndScrollUp
ScrollUp:
	ld		b,5
LooperUp:
	ld		a,(hl)
	ld		(de),a
	inc		hl
	inc		de
	djnz 	LooperUp
	ret

EndScrollUp:
	pop		hl
	dec		(hl)
	ret

DisplayHandler:
	ld		a,(Pos)
	ld		h,a
	ld		l,$07
	call	MUL_HL
	ld		de,$01
	add		hl,de
	ld		(_penRow),hl
	ld		a,$02
	ld		(_penCol),a
	call	ClearLine
	ld		a,$02
	ld		(_penCol),a
	ld		a,(_OP1+1)
	ld		b,a
	ld		hl,_OP1+2
	call	_vputsn
	ret

Descriptions:
	call	_push_OP1
	ld		hl,$FCF8
	ld		c,$07
ClearLines:
	push	hl
	call	Clearer	
	pop		hl
	ld		de,$10
	add		hl,de
	dec		c
	ld		a,c
	cp		0
	jr		nz,ClearLines
	jr		BasicProgram
Clearer:
	ld		b,$08
ClearLoop:
	ld		(hl),$00
	inc		hl
	djnz 	ClearLoop
	ret

BasicProgram:
	ld		hl,$0F2C
	ld		(_penCol),hl
	rst 	10h
	call	_checkbasic
	jr		z,AsmProgram
	ld		hl,Basic_Str
	call	_vputs
	jr		Get_Size

AsmProgram:
	ld		a,b
	ld		h,d
	ld		l,e
	call	_load_ram_ahl
	ld		de,$04
	add		hl,de
	ld		a,(hl)
	cp		$00
	jr		nz,RegAsm
	push	hl
	ld		de,$06
	add		hl,de
	ld		e,(hl)
	inc		hl
	ld		d,(hl)
	ex		de,hl
	ld		de,$D748
	sbc		hl,de
	pop		de
	add		hl,de
	call	_vputs
	jr		Get_Size
RegAsm:
	ld		hl,Asm_Str
	call	_vputs
Get_Size:
	ld		hl,$1D2C
	ld		(_penCol),hl	
	rst		10h
	ld		a,b
	ex		de,hl
	call 	_get_word_ahl		;Get program size
	ex		de,hl
	xor		a
	call	DispHL				;display hl
	call	_pop_OP1			;recall OP1
	ret

InvertCursor:
	call	Find_Vid_Addr
	call	InvertH
	ret

InvertH:
	ld		c,7
InvertBar:
	call	Inverter	
	ld		de,11
	add		hl,de
	dec		c
	ld		a,c
	cp		0
	jr		nz,InvertBar
	ret
Inverter:
	ld		b,5
InvertLooper:
	ld		a,(hl)
	cpl
	ld		(hl),a
	inc		hl
	djnz 	InvertLooper
	ret

ClearLine:
	ld		hl,Clear_Str
	call	_vputs	
	ld		hl,Clear_Str
	call	_vputs
	ret

Find_Vid_Addr:
	ld      a,(Pos)				;load the position 1-7
	ld      h,a					;into h
	ld      l,$70				;
	call    MUL_HL				;
	ld      de,$FC10			;add offset
	add     hl,de			
	ret

;**************************Invert 7 lines from hl by Andreas Ess****************
Invert:
	ld      b,$70
InvertLoop:
	ld      a,(hl)
	cpl
	ld      (hl),a
	inc     hl
	djnz	InvertLoop
	ret

;********************Display HL Routine By Joshua Seagoe**************
DispHL:
	ld		de,-1
	ld		(_curRow),de
	call	_dispahl
	dec		hl
	call	_vputs
	ld		hl,Bytes_Str
	call	_vputs
	ret

RestoreMSE:
	call	_push_OP1
	ld		hl,VatPtr
	rst		20h
	rst		10h
	ld		(hl),$12
	call	_pop_OP1
	ret

;************End Routine***************

Exit:
	call	RestoreMSE
	call	_clrScrn
	call	_jforce

;********************Strings used in program*********************
Clear_Str:
	.db "                  ",0

FreeMem_Str:
	.db " Free Memory: ",0

Name_Str:
	.db " Name: ",0

Size_Str:
	.db " Size: ",0
NoProgs:
	.db "No Programs!",0

Asm_Str:
	.db "Asm Program",0

Basic_Str:
	.db "Basic Program",0

AbsFirst:
	.db $12,$01,$00

VatPtr:
	.db $00,$03,"mse",0

PrgmName:
	.db "MS-Enhanced            by Chicane                v1.3",0

Bytes_Str:
	.db "  bytes          ",0

;--------------Vars used in program------>
Pos:
	.db	0

Items:
	.db 0
.end

